// ======================================================================
//
// Quaternion.cpp
// Gage Galinger
//
// Copyright 1998 Origin Systems Inc
//
//  NOTES:  This quaternion class handles rotations only.  No translation 
//  information is stored.  In order to be truly useful for handling 
//  hierarchical models and slerping keyframes, this class needs to 
//  have a translation vector and transformation stack added to it.
//  In its current state it should only be used for the handy rotation
//  transforms it provides.
//  The code is currently %100 unoptimized.
// ======================================================================


#ifndef QUATERNION_H
#define QUATERNION_H

class Xform;
class Vector;

class Quaternion
{
public:

	static const Quaternion identity;

public:

	enum
	{
		SAVE_SIZE = 4
	};

	struct Save
	{
		real save[SAVE_SIZE];
	};

public:

	real w;
	real x;
	real y;
	real z;

public:
	//-- Construction/Destruction
	Quaternion(void);
	Quaternion(const Quaternion& Q);
	Quaternion(const Xform& X);
	Quaternion(real radians, const Vector& V);
	Quaternion(real radians, real ax, real ay, real az);
	~Quaternion(void);

	//-- Utility
	void loadIdentity(void);
	void multiply(const Quaternion& Q);

	//-- Rotation
	void setRotation(const Quaternion& Q);
	void setRotation(const Xform& X);
	void setRotation(real radians, const Vector& V);
	void setRotation(real radians, real ax, real ay, real az);

	void addRotation(const Quaternion& Q);
	void addRotation(const Xform& X);
	void addRotation(real radians, const Vector& V);
	void addRotation(real radians, real ax, real ay, real az);

	void getXform(Xform* X) const;
	void getAxis(real* radians, real* ax, real* ay, real* az) const;

	void slerp (const Quaternion& a, const Quaternion& b, real alpha, real spin = 0);

	// support routines
	void   writeSave(Save *save);
	void   readSave(const Save &save);
};

inline void Quaternion::writeSave(Save *save)
{
	save->save[0] = w;
	save->save[1] = x;
	save->save[2] = y;
	save->save[3] = z;
}

inline void Quaternion::readSave(const Save &save)
{
	w = save.save[0];
	x = save.save[1];
	y = save.save[2];
	z = save.save[3];
}

#endif

/*
// ======================================================================
//
// file: $/prophecy/src/engine/3d/quatern.h
// (c) origin systems, 1996
// author: hugh david
// description: quaternion class
// notes:
//  - deals only with unit quaternions
// to do:
//  - optimize
//  - Quaternion to Xform function
//  - Xform to Quaternion function
//
// ======================================================================

#ifndef  QUATERNION_H
#define  QUATERNION_H

// ======================================================================

#include "first.h"
#include "vector.h"
class    Xform;

// ======================================================================

class Quaternion
{
	friend class Xform;

public:

	enum
	{
		SAVE_SIZE = 4
	};

	struct Save
	{
		real save[SAVE_SIZE];
	};

public:

	real w, x, y, z;

public:

	// constructors
	Quaternion(void);
	Quaternion(const Quaternion &q);
	explicit Quaternion(const Vector &v, real w=CONST_REAL(0));
	explicit Quaternion(const Xform &xform);
	Quaternion(real newW, real newX, real newY, real newZ);

	// operators
	Quaternion &operator =(const Quaternion &q);

	// general functions
	void conjugate(void);
	void mul(const Quaternion &ql, const Quaternion &qr);
	void rotation(const Vector &v, real angle);
	void transform(const Vector&src, Vector *dst) const;

	// support routines
	void   writeSave(Save *save);
	void   readSave(const Save &save);

//private:

	// keep private until fully tested:
	void slerp (const Quaternion& a, const Quaternion& b, real alpha, real spin = 0);
};

// ======================================================================

inline Quaternion::Quaternion(void)
{
	w = CONST_REAL(1);
	x = 0;
	y = 0;
	z = 0;
}

// ----------------------------------------------------------------------

inline Quaternion::Quaternion(const Quaternion &q)
{
	w = q.w;
	x = q.x;
	y = q.y;
	z = q.z;
}

// ----------------------------------------------------------------------
// don't use this unless you know what you're doing in quaternion maths.
// this constructor is used by the transform function.

inline Quaternion::Quaternion (const Vector& v, real newW)
{
	w = newW;
	x = v.x;
	y = v.y;
	z = v.z;
}

// ----------------------------------------------------------------------
//
// don't use this unless you know what you're doing in quaternion maths.
// this constructor is used by the complement operator.

inline Quaternion::Quaternion(real newW, real newX, real newY, real newZ)
{
	w = newW;
	x = newX;
	y = newY;
	z = newZ;
}

// ----------------------------------------------------------------------

inline Quaternion &Quaternion::operator =(const Quaternion &q)
{
	if (&q != this)
	{
		w = q.w;
		x = q.x;
		y = q.y;
		z = q.z;
	}

	return *this;
}

// ----------------------------------------------------------------------
// multiplication operator:
// note:
//  - quaternion multiplication is NOT commutative

inline void Quaternion::mul(const Quaternion &ql, const Quaternion &qr)
{
	real ww, xx, yy, zz;

	// create into temp variables so we can use *this as one of the arguments
	ww = ql.w * qr.w - ql.x * qr.x - ql.y * qr.y - ql.z * qr.z;
	xx = ql.w * qr.x + ql.x * qr.w + ql.y * qr.z - ql.z * qr.y;
	yy = ql.w * qr.y + ql.y * qr.w + ql.z * qr.x - ql.x * qr.z;
	zz = ql.w * qr.z + ql.z * qr.w + ql.x * qr.y - ql.y * qr.x;

	w = ww;
	x = xx;
	y = yy;
	z = zz;
}

// ----------------------------------------------------------------------
// conjugate operator

inline void Quaternion::conjugate(void)
{
	x = -x;
	y = -y;
	z = -z;
}

// ----------------------------------------------------------------------
// routine to transform a Vector by a Quaternion

inline void Quaternion::transform(const Vector &src, Vector *dst) const
{
	Quaternion qc(*this), r;

	// transform is effected by (q * v) * conjugate(q)
	qc.conjugate();
	r.mul(*this, Quaternion(src));
	r.mul(r, qc);

	dst->x = r.x;
	dst->y = r.y;
	dst->z = r.z;
}

// ----------------------------------------------------------------------

inline void Quaternion::writeSave(Save *save)
{
	save->save[0] = w;
	save->save[1] = x;
	save->save[2] = y;
	save->save[3] = z;
}

// ----------------------------------------------------------------------

inline void Quaternion::readSave(const Save &save)
{
	w = save.save[0];
	x = save.save[1];
	y = save.save[2];
	z = save.save[3];
}

// ======================================================================

#endif


*/